<?php
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST");
header("Content-Type: application/json");

require_once 'vendor/autoload.php';
require_once 'config.php';

// Database and config parameters
$host = DATABASE_HOST;
$dbname = DATABASE_NAME;
$username = DATABASE_USER;
$password = DATABASE_PASSWORD;
$base_url = BASE_URL;
$dpo_endpoint = DPO_ENDPOINT;
$company_token = COMPANY_TOKEN;

$companyRef = COMPANY_REF;
$serviceType = SERVICE_TYPE;
// Get request data
$data = json_decode(file_get_contents("php://input"), true);
if (!$data) {
    echo json_encode(["error" => "Invalid input data."]);
    exit;
}

$amount = $data["amount"] ?? null;
$email = $data["email"] ?? null;
$firstName = $data["firstName"] ?? null;
$lastName = $data["lastName"] ?? null;
$mobileNumber = $data["phone"] ?? null;
$subscriptionPeriod = $data["subscriptionPeriod"] ?? null;
$isRecurring = $data["isRecurring"] ?? '0';

if (!$amount || !$email || !$firstName || !$lastName || !$mobileNumber) {
    echo json_encode(["error" => "Missing required fields."]);
    exit;
}

// Validate and calculate expiry date based on subscription period (in days)
if (!is_numeric($subscriptionPeriod) || $subscriptionPeriod <= 0) {
    echo json_encode(["error" => "Invalid subscription period."]);
    exit;
}

// Calculate expiry date by adding the subscription period (in days) to the current date
$expiryDate = date('Y-m-d', strtotime("+$subscriptionPeriod days")); // Add subscription period to current date

// Success and failure URLs (without $transToken for now)
$redirectURL = $base_url . "/membership/payment_success.php";

$backURL = $base_url . "/membership/payment_failed.php";

// Prepare XML payload
$xmlData = '<?xml version="1.0" encoding="utf-8"?>
<API3G>
  <CompanyToken>' . htmlspecialchars($company_token) . '</CompanyToken>
  <Request>createToken</Request>
  <Transaction>
    <PaymentAmount>' . htmlspecialchars($amount) . '</PaymentAmount>
    <PaymentCurrency>ZMW</PaymentCurrency>
    <CompanyRef>' . htmlspecialchars($companyRef) . '</CompanyRef>
    <RedirectURL>' . htmlspecialchars($redirectURL) . '</RedirectURL>
    <BackURL>' . htmlspecialchars($backURL) . '</BackURL>
    <customerFirstName>' . htmlspecialchars($firstName) . '</customerFirstName>
    <customerLastName>' . htmlspecialchars($lastName) . '</customerLastName>
    <customerEmail>' . htmlspecialchars($email) . '</customerEmail>
    <customerPhone>' . htmlspecialchars($mobileNumber) . '</customerPhone>
    <CompanyRefUnique>0</CompanyRefUnique>
    <AllowRecurrent>' . $isRecurring . '</AllowRecurrent>
    <PTL>5</PTL>
  </Transaction>
  <Services>
    <Service>
      <ServiceType>' . htmlspecialchars($serviceType) . '</ServiceType>
      <ServiceDescription>Payment for services</ServiceDescription>
      <ServiceDate>' . date('Y-m-d H:i:s') . '</ServiceDate>
    </Service>
  </Services>
</API3G>';

// cURL request to DPO
$ch = curl_init();
curl_setopt($ch, CURLOPT_URL, $dpo_endpoint);
curl_setopt($ch, CURLOPT_POST, true);
curl_setopt($ch, CURLOPT_POSTFIELDS, $xmlData);
curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
curl_setopt($ch, CURLOPT_HTTPHEADER, ["Content-Type: application/xml"]);

$response = curl_exec($ch);
$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
curl_close($ch);

file_put_contents('request.txt', $xmlData);
file_put_contents('response.txt', $response);

// Validate response
if (!$response || $httpCode !== 200) {
    echo json_encode(["error" => "Failed to connect to DPO."]);
    exit;
}

// Convert XML response to an object
$xml = simplexml_load_string($response);
if (!$xml) {
    echo json_encode(["error" => "Invalid response from DPO."]);
    exit;
}

// Extract transaction token
$transToken = isset($xml->TransToken) ? (string)$xml->TransToken : '';

if (!empty($transToken)) {
    // Save the transaction in the database
    try {
        // Create a database connection
        $pdo = new PDO("mysql:host=$host;dbname=$dbname", $username, $password);
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        // Prepare SQL query to insert transaction details
        $stmt = $pdo->prepare("
            INSERT INTO transactions (company_ref, amount, email, first_name, last_name, phone, subscription_period, is_recurring, trans_token, company_token, transaction_status, expiry_date)
            VALUES (:company_ref, :amount, :email, :first_name, :last_name, :phone, :subscription_period, :is_recurring, :trans_token, :company_token, :transaction_status, :expiry_date)
        ");

        // Bind parameters and execute the query
        $stmt->execute([
            ':company_ref' => $companyRef,
            ':amount' => $amount,
            ':email' => $email,
            ':first_name' => $firstName,
            ':last_name' => $lastName,
            ':phone' => $mobileNumber,
            ':subscription_period' => $subscriptionPeriod,
            ':is_recurring' => $isRecurring,
            ':trans_token' => $transToken,
            ':company_token' => $company_token,
            ':transaction_status' => 'initiated',  // Changed from 'status' to 'transaction_status'
            ':expiry_date' => $expiryDate  // Dynamically calculated expiry date
        ]);

        // Send back the transToken to the frontend
        echo json_encode([
            "transToken" => $transToken
        ]);
        exit; // Ensure no further processing occurs after response
    } catch (PDOException $e) {
        echo json_encode(["error" => "Failed to save transaction to database: " . $e->getMessage()]);
        exit;
    }
} else {
    echo json_encode(["error" => "Transaction token not found."]);
    exit;
}
?>
