<?php
require_once 'vendor/autoload.php';
require_once 'config.php';
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;
use PHPMailer\PHPMailer\SMTP;

$host = DATABASE_HOST;
$dbname = DATABASE_NAME;
$username = DATABASE_USER;
$password = DATABASE_PASSWORD;
$dpo_endpoint = DPO_ENDPOINT;
$company_token = COMPANY_TOKEN;



// Get today's date
$today = date('Y-m-d');

// Log file path
$logDir = __DIR__ . "/verifyTransfers";
$logFile = $logDir . "/log.txt";

// Check if the directory exists, if not, create it
if (!is_dir($logDir)) {
    mkdir($logDir, 0777, true); // 0777 gives read, write, and execute permissions to all
}

// Ensure the log file is writable (it will be created if it doesn't exist)
if (!is_writable($logFile)) {
    touch($logFile); // Create the file if it doesn't exist
}

try {
    // Create PDO connection
    $pdo = new PDO("mysql:host=$host;dbname=$dbname", $username, $password);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

    // SQL to fetch transactions that need to be re-verified
    $sql = "SELECT * FROM transactions WHERE DATE(created_at) = :today AND transaction_status = 'Transaction RENEWED'";
    $stmt = $pdo->prepare($sql);
    $stmt->execute(['today' => $today]);
    $transactions = $stmt->fetchAll(PDO::FETCH_ASSOC);

    // Check if there are any transactions to verify
    if (count($transactions) > 0) {
        foreach ($transactions as $transaction) {
            // Extract transaction details
            $token = $transaction['trans_token'];
            $companyToken = $transaction['company_token']; // Corrected: Fetch company_token from the current transaction
            // DPO Verification Endpoint
            $endpoint = $dpo_endpoint;

            // XML Payload for verification
            $xmlData = '<?xml version="1.0" encoding="utf-8"?>
            <API3G>
                <CompanyToken>' . $companyToken . '</CompanyToken>
                <Request>verifyToken</Request>
                <TransactionToken>' . $token . '</TransactionToken>
            </API3G>';

            // cURL request to DPO
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $endpoint);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $xmlData);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, ["Content-Type: application/xml"]);

            $response = curl_exec($ch);
            curl_close($ch);

            // Convert XML response to object
            $xml = simplexml_load_string($response);

            // Extract verification results
            $resultCode = (string) $xml->Result;

            // if the result code is 000 (success) send an email to the user and admin with the transaction details

            if ($resultCode === '000') {
                // Send email to user
                $mail = new PHPMailer(true);
                try {
                    // SMTP configuration
                    $mail->isSMTP();
                    $mail->Host = SMTP_HOST;
                    $mail->SMTPAuth = true;
                    $mail->Username = SMTP_USERNAME;
                    $mail->Password = SMTP_PASSWORD;
                    $mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;
                    $mail->Port = SMTP_PORT;

                    // Email details
                    $mail->setFrom(SMTP_USERNAME, 'Buying Property in Zambia');
                    $mail->addAddress($transaction['email']);
                    $mail->Subject = TRANSACTION_VERIFICATION_SUBJECT;
                    $mail->Body = TRANSACTION_VERIFICATION_BODY;
                    $mail->send();
                    error_log('Email sent successfully to ' . $transaction['email']);

                    /// send to admin as well
                    $mail->addAddress(ADMIN_EMAIL_ADDRESS);
                    $mail->Subject = SUBSCRIPTION_SUBJECT_RENEWAL;
                    // new subcription
                    $mail->Body = SUBSCRIPTION_BODY_RENEWAL;
                    $mail->send();
                    error_log('Email sent successfully to ' . $transaction['email']);
                } catch (Exception $e) {
                    error_log("Email could not be sent. Mailer Error: {$mail->ErrorInfo}");
               
                }
            }

            // Check if the result code is 000 (success)
            $resultExplanation = (string) $xml->ResultExplanation;

            // Store the result back into the database
            updateTransactionVerification($pdo, $transaction['id'], $resultExplanation);
        }
    } else {
        echo "No transactions need verification today.";
    }
} catch (PDOException $e) {
    echo "Error: " . $e->getMessage();
}

function updateTransactionVerification($pdo, $transactionId, $transaction_status) {
    // Update the transaction with the result of the verification
    $sql = "UPDATE transactions 
            SET   transaction_status = :transaction_status
            WHERE id = :transaction_id";
    $stmt = $pdo->prepare($sql);
    $stmt->execute([
        'transaction_status' => $transaction_status,   
        'transaction_id' => $transactionId
    ]);
}
?>
